#!/bin/bash
#
# Simple script to delete old Catalina logs in multiple Tomcat installations.
# Part of tomcat6-addons package.

DEFAULT="/etc/default"				# Defaults folder
FIND="/usr/bin/find"				# find command call
GREP="/bin/grep"                        	# grep command call
LOGFILE_DAYS_DEFAULT=14				# Default preserve time
SED="/bin/sed"                          	# sed command call
TOMCAT="tomcat6"                        	# Tomcat's name (at all)
TOMCAT_BASEDIR="/var/opt/tomcat6"               # Common root for Tomcats
TOMCAT_LIST=`ls -1 $DEFAULT/$TOMCAT.* 2>/dev/null | $SED "s/^.*\/\(.*\)$/\1/"`
TOMCAT_NAMES=$(echo -e "$TOMCAT_LIST" | $SED "s/^$TOMCAT\.//" | $GREP -v "^[0-9]*$")
XARGS="/usr/bin/xargs"				# xargs command call

MSG_ROOTNEED="Run as root, please."

# Root privileges needed.
if [ ! $UID -eq 0 ]; then echo -e "$MSG_ROOTNEED" >&2; exit $EXIT_ERR; fi

# Enumerating valid Tomcat instances by name.
for selected_tomcat in $TOMCAT_NAMES
do
    # Logdir present?
    if [ -d "$TOMCAT_BASEDIR/$selected_tomcat/logs" ]; then
	# Try to get LOGFILE_DAYS from defaults.
	if [ -e "$DEFAULT/$TOMCAT.$selected_tomcat" ]; then
	    LOGFILE_DAYS=$(cat "$DEFAULT/$TOMCAT.$selected_tomcat" | $SED -n "s/^LOGFILE_DAYS=\([0-9]*\)$/\1/p"); #"
	    LOGFILE_DAYS=${LOGFILE_DAYS:-$LOGFILE_DAYS_DEFAULT}
	    # Delete *.log from logdir when oldest than $LOGFILE_DAYS.
            $FIND "$TOMCAT_BASEDIR/$selected_tomcat/logs"/ -name \*.log \
	         -mtime +$LOGFILE_DAYS -print0 | \
    	         $XARGS --no-run-if-empty -0 rm --
	fi
    fi
done

# That's all, folks! :-)
