#!/bin/bash
#
# Simple script to watch Java/Tomcat out of memory errors or dead Tomcats
# within multiple Tomcat installations. Run as a cron job, once per minute.
# Part of mimoza-tomcat5.5 package.

exit 0

AWK="/usr/bin/awk"				# awk command call
CUT="/usr/bin/cut"				# cut command call
DEAD="engine is not running, but pid file exists"	# Error message to watch
DEBUG=1						# Enable debug messages and logs
DEFAULT="/etc/default"				# Defaults directory
ERROR="java.lang.OutOfMemoryError"		# Error message to watch
EXIT_ERR="1"					# Unsuccessfull exit signal
GREP="/bin/grep"				# grep command call
INIT="/etc/init.d"				# Init script directory
LOGBASE="/var/log/tomcat5.5"			# Logs directory
LOGERROR="restart.`date '+%Y-%m-%d'`.log"	# Own logfile
LOGFILE1="localhost.`date +%Y-%m-%d`.log"	# Watched logfile #1
LOGFILE2="catalina.`date +%Y-%m-%d`.log"	# Watched logfile #2
LOGFILEGC="gc.log"				# Watched garbage collector logfile
LOGFILENS="nanosecs.`date '+%Y-%m-%d'`.log"	# Additional GC monitoring logfile
LOGGROUP="www-data"				# Own logfile Liux group
LOGOWNER="webadmin"				# Own logfile Linux user
LOGPOINTER="/var/log/tomcat5.5"			# Log pointers directory
LOGRIGHTS="640"					# Own logfile Linux rights
MINGCTIME="600"					# GC check grace time (secs)
#MINGCTIME="0"					#     set 0 to disable
MAXGCTIME="300000000"				# Max GC in 0,1 usecs(!) 30 secs
#MAXGCTIME="0"					#     set 0 to disable
PIDBASE="/var/lib/tomcat5.5/temp"		# PID files directory
PS="/bin/ps"					# ps command call
SED="/bin/sed"					# sed command call
SLEEPTIME="5"					# Secs between restart and status
TAIL="/usr/bin/tail"				# tail command call
TOMCAT="tomcat5.5"				# Tomcat vesrion
TOMCAT_LIST=`ls -1 $DEFAULT/$TOMCAT.* 2>/dev/null | sed "s/^.*\/\(.*\.\)\(.*\)$/\2/"`
TOMCAT_SCRIPT="/usr/local/sbin/tomcat.sh"	# Multiple Tomcat handler script

# A beavatkozo scriptnek leteznie kell!

if [ ! -x $TOMCAT_SCRIPT ]; then
    echo -e "$0: Missing $TOMCAT_SCRIPT - exiting.\n"
    exit $EXIT_ERR
fi

# Sorbaveszi a letezo Tomcat-eket, es megvizsgalja:
# 1. a PID file konzisztenciat, ha elterest talal, leallit.
# 2. a statust. Ha megtalalja a $DEAD szoveget, indit.
# 3. a logok meg nem vizsgal sorait. Ha megtalalja az $ERROR szoveget, ujraindit.
# A beavatkozasokat logolja.

for current_tomcat in $TOMCAT_LIST
do
    # Tomcat specifikus beallitasok felolvasasa
    if [ -f $DEFAULT/$TOMCAT.$current_tomcat ]; then
	. $DEFAULT/$TOMCAT.$current_tomcat
    fi 
    # Sajat logfile ellenorzese
    logfile="$LOGBASE/$current_tomcat/$LOGERROR"
    if [ ! -f  $logfile ]; then
	touch $logfile; chown $LOGOWNER:$LOGGROUP $logfile; chmod $LOGRIGHTS $logfile
    fi
    # Esetleges PID file inkonzisztencia kezelese
    pid_real=`$PS aux | $GREP 'java' | $GREP "/$current_tomcat/" | $GREP -v "$GREP" | $AWK '{print $2}'`
    pid_file=`cat $PIDBASE/$current_tomcat/$current_tomcat.pid 2>/dev/null`
    # Csak ha leteznek ezek a PID-ek
    if [ ! -z "$pid_real" -a ! -z "$pid_file" ]; then
	# Az inkonzisztens Tomcet-et le kell allitani
	if [ $pid_real -ne $pid_file ]; then
    	    echo "$0: Killing inconsistent $current_tomcat at PID $pid_real"
	    echo "`date '+%Y.%m.%d %H:%M:%S'` Killing inconsistent $current_tomcat at PID $pid_real" >>$LOGBASE/$current_tomcat/$LOGERROR
	    kill -9 "$pid_real"
    	    sleep $SLEEPTIME
	fi
    fi
    # Esetleges dead Tomcat (process nincs, de pidfile van) ujrainditasa
    status=`$TOMCAT_SCRIPT $current_tomcat status`
    if [ ! -z "`$TOMCAT_SCRIPT $current_tomcat status | grep "$DEAD"`" ]; then
        echo "$0: Restarting dead $current_tomcat"
	echo "`date '+%Y.%m.%d %H:%M:%S'` dead PID: $pid_file"
	echo "`date '+%Y.%m.%d %H:%M:%S'` Restarting dead $current_tomcat" >>$LOGBASE/$current_tomcat/$LOGERROR
        $TOMCAT_SCRIPT $current_tomcat start
        sleep $SLEEPTIME
        $TOMCAT_SCRIPT $current_tomcat status
    else
	# Tomcat rendben, logvizsgalat kovetkezik
	# Meg nem tortent beavatkozas
	restarted=0
	for logfile in $LOGFILE1 $LOGFILE2 $LOGFILEGC
	do
	    pointerfile="$LOGPOINTER/$current_tomcat/"`echo "$logfile" | $CUT -d '.' -f 1`".ptr"
	    # Pointer elkeszitese, ha meg nem lenne
	    if [ ! -f "$pointerfile" ]; then
		echo 0 > $pointerfile; chown $LOGOWNER:$LOGGROUP $pointerfile; chmod $LOGRIGHTS $pointerfile
	    fi
	    # Ha nincs logfile, a pointert nullazzuk es keszen vagyunk
	    if [ ! -f "$LOGBASE/$current_tomcat/$logfile" ]; then
		echo 0 > $pointerfile
	    else
		# Pointer frissitese, logfile atnezese, ujrainditas, ha kell
	        pointer=`cat $pointerfile`
		current=`cat $LOGBASE/$current_tomcat/$logfile | wc -l`; (( current++ ))
		echo "$current" > $pointerfile
		# Ellentmondas eseten nullazzuk a pointert
		if [ "$pointer" -gt "$current" ]; then pointer="0"; fi
		# Nem kell beavatkozni, ha mar megtortent
		if [ $restarted -eq 0 ]; then
		    if [ $logfile = $LOGFILEGC -a $MAXGCTIME -gt 0 ]; then
		    # Beavatkozas, ha GC log szerint tul sokaig tart a GC
			nanosecs=0
			$TAIL -q -n +$pointer $LOGBASE/$current_tomcat/$logfile 2>/dev/null | while read line
			do
			    gctime=`echo $line | $SED "s/\([0-9]*\).*/\1/"`
			    let "nanosecs = nanosecs + `echo $line | $SED "s/.* \([0-9]*\)\.\([0-9]*\) secs\]$/10\#\1\2/"`"
			    # Debug logolas, ha szukseges
			    if [ "$DEBUG" -eq 1 ]; then
				if [ ! -f "$LOGBASE/$current_tomcat/$LOGFILENS" ]; then
				    touch $LOGBASE/$current_tomcat/$LOGFILENS 
				    chown $LOGOWNER:$LOGGROUP $LOGBASE/$current_tomcat/$LOGFILENS 
				    chmod $LOGRIGHTS $LOGBASE/$current_tomcat/$LOGFILENS
				fi
				echo $gctime $nanosecs >>$LOGBASE/$current_tomcat/$LOGFILENS
			    fi
			    # Csak akkor, ha mar elmult a grace time es tul sok a GC time
			    if [ $gctime -gt $MINGCTIME -a $nanosecs -ge $MAXGCTIME ]; then
				# Jelzes: beavatkozas tortent
            			echo "$0: GC time $nanosecs reached the limit ($MAXGCTIME) - restarting $current_tomcat"
		    		echo "`date '+%Y.%m.%d %H:%M:%S'` $logfile pointer: $pointer current: $current"
				# Elkonyveles a restart logban
				echo "`date '+%Y.%m.%d %H:%M:%S'` GC time $nanosecs reached the limit ($MAXGCTIME) - restarting $current_tomcat" >>$LOGBASE/$current_tomcat/$LOGERROR
				# Tomcat restart
            			$TOMCAT_SCRIPT $current_tomcat restart
            			sleep $SLEEPTIME
            			$TOMCAT_SCRIPT $current_tomcat status
				# Ezzel a Tomcat-tel mar nem kell foglalkozni
				break 2
			    fi
			done
		    else
		    # Beavatkozas, ha a logreszletben hibajelzes van
            		if [ ! -z "`$TAIL -q -n +$pointer $LOGBASE/$current_tomcat/$logfile 2>/dev/null | $GREP -i "$ERROR"`" ]; then
			    # Jelzes: beavatkozas tortent
			    restarted=1
            		    echo "$0: $ERROR detected - restarting $current_tomcat"
		    	    echo "`date '+%Y.%m.%d %H:%M:%S'` $logfile pointer: $pointer current: $current"
			    # Elkonyveles a restart logban
			    echo "`date '+%Y.%m.%d %H:%M:%S'` $ERROR detected - restarting $current_tomcat" >>$LOGBASE/$current_tomcat/$LOGERROR
			    # Tomcat restart
            		    $TOMCAT_SCRIPT $current_tomcat restart
            		    sleep $SLEEPTIME
            		    $TOMCAT_SCRIPT $current_tomcat status
                    	    # Pointer tovabbvitele a restart utani allapotra
                    	    current=`cat $LOGBASE/$current_tomcat/$logfile | wc -l`; (( current++ ))
                    	    echo "$current" > $pointerfile
			fi
		    fi
		fi
	    fi
	done
    fi
done
